@@ -65,7 +65,10 @@
 #include "ratectl.hh"
 #include "seqencoder.hh"
 #include "mpeg2coder.hh"
+
+#include "yuv4mpeg.h"
 #include "format_codes.h"
+#include "mpegconsts.h"
 
 #ifdef HAVE_ALTIVEC
 /* needed for ALTIVEC_BENCHMARK and print_benchmark_statistics() */
@@ -74,9 +77,226 @@
 
 
 
+/**************************
+ *
+ * Derived class for File (actually: pipe and FIFO) based bitstream output
+ *
+ *************************/
 
 
 
+class FILE_StrmWriter : public ElemStrmWriter
+{
+public:
+    FILE_StrmWriter( EncoderParams &encoder, const char *ofile_ptr ); 
+    virtual ~FILE_StrmWriter();       
+    void PutBits( uint32_t val, int n);
+    void FrameBegin();
+    void FrameFlush();
+    void FrameDiscard();
+    
+private:
+    FILE *outfile;
+};
+
+
+
+
+FILE_StrmWriter::FILE_StrmWriter( EncoderParams &encparams, const char *outfilename ) :
+	ElemStrmWriter( encparams )
+{
+	/* open output file */
+	if (!(outfile=fopen(outfilename,"wb")))
+	{
+		mjpeg_error_exit1("Couldn't create output file %s",outfilename);
+	}
+}
+
+FILE_StrmWriter::~FILE_StrmWriter()
+{
+	fclose( outfile );
+}
+
+/**************
+ *
+ * Write rightmost n (0<=n<=32) bits of val to outfile 
+ *
+ *************/
+void FILE_StrmWriter::PutBits(uint32_t val, int n)
+{
+	val = (n == 32) ? val : (val & (~(0xffffffffU << n)));
+	while( n >= outcnt )
+	{
+		outbfr = (outbfr << outcnt ) | (val >> (n-outcnt));
+		putc( outbfr, outfile );
+		n -= outcnt;
+		outcnt = 8;
+		++bytecnt;
+	}
+	if( n != 0 )
+	{
+		outbfr = (outbfr<<n) | val;
+		outcnt -= n;
+	}
+}
+
+
+void FILE_StrmWriter::FrameBegin()
+{
+}
+
+void FILE_StrmWriter::FrameFlush()
+{
+}
+
+void FILE_StrmWriter::FrameDiscard()
+{
+}
+
+
+/**************************
+ *
+ * Derived class for File (actually: pipe and FIFO) based bitstream output
+ *
+ *************************/
+
+
+class Y4MPipeReader : public PictureReader
+{
+public:
+    Y4MPipeReader( EncoderParams &encparams, int pipe_fd );
+    ~Y4MPipeReader() {}
+    void StreamPictureParams( MPEG2EncInVidParams &strm );
+protected:
+    bool LoadFrame( );
+private:
+    int PipeRead(  uint8_t *buf, int len);
+
+    int pipe_fd;
+};
+ 
+
+Y4MPipeReader::Y4MPipeReader( EncoderParams &encparams, int istrm_fd ) :
+    PictureReader( encparams ),
+    pipe_fd( istrm_fd )
+{
+}
+
+
+/****************************************
+ *
+ * Initialise the reader and return the parameter of the video stream
+ * to be encoded.
+ *
+ * WARNING: This routine must run before encoder parameters are defined.
+ * TODO: Reader should be constructed before EncoderParams and this
+ * routine invoked from EncoderParams constructor...
+ *
+ *****************************************/
+
+void Y4MPipeReader::StreamPictureParams( MPEG2EncInVidParams &strm )
+{
+   int n;
+   y4m_ratio_t sar;
+   y4m_stream_info_t si;
+
+   y4m_init_stream_info (&si);  
+   if ((n = y4m_read_stream_header (pipe_fd, &si)) != Y4M_OK) {
+       mjpeg_log( LOG_ERROR, 
+                  "Could not read YUV4MPEG2 header: %s!",
+                  y4m_strerr(n));
+      exit (1);
+   }
+
+   strm.horizontal_size = y4m_si_get_width(&si);
+   strm.vertical_size = y4m_si_get_height(&si);
+   strm.frame_rate_code = mpeg_framerate_code(y4m_si_get_framerate(&si));
+   strm.interlacing_code = y4m_si_get_interlace(&si);
+
+   /* Deduce MPEG aspect ratio from stream's frame size and SAR...
+      (always as an MPEG-2 code; that's what caller expects). */
+   sar = y4m_si_get_sampleaspect(&si);
+   strm.aspect_ratio_code = 
+       mpeg_guess_mpeg_aspect_code(2, sar, 
+                                   strm.horizontal_size, 
+                                   strm.vertical_size);
+   if(strm.horizontal_size <= 0)
+   {
+       mjpeg_error_exit1("Horizontal size from input stream illegal");
+   }
+   if(strm.vertical_size <= 0)
+   {
+       mjpeg_error("Vertical size from input stream illegal");
+   }
+}
+
+/*****************************
+ *
+ * LoadFrame - pull in the image data planes from pipe (planar YUV420)
+ *
+ * RETURN: true iff EOF or ERROR
+ *
+ ****************************/
+
+bool Y4MPipeReader::LoadFrame( )
+{
+   y4m_frame_info_t fi;
+   int h,v,y;
+   y4m_init_frame_info (&fi);
+   int buffer_slot = frames_read % input_imgs_buf_size;
+
+
+   if ((y = y4m_read_frame_header (pipe_fd, &fi)) != Y4M_OK) 
+   {
+       if( y != Y4M_ERR_EOF )
+           mjpeg_log (LOG_WARN, 
+                      "Error reading frame header (%d): code%s!", 
+                      frames_read,
+                      y4m_strerr (y));
+       return true;
+      }
+      
+   v = encparams.vertical_size;
+   h = encparams.horizontal_size;
+   int i;
+   for(i=0;i<v;i++)
+   {
+       if( PipeRead(input_imgs_buf[buffer_slot][0]+i*encparams.phy_width,h)!=h)
+           return true;
+   }
+   lum_mean[buffer_slot] = LumMean(input_imgs_buf[buffer_slot][0] );
+   v = encparams.vertical_size/2;
+   h = encparams.horizontal_size/2;
+   for(i=0;i<v;i++)
+   {
+       if(PipeRead(input_imgs_buf[buffer_slot][1]+i*encparams.phy_chrom_width,h)!=h)
+           return true;
+   }
+   for(i=0;i<v;i++)
+   {
+       if(PipeRead(input_imgs_buf[buffer_slot][2]+i*encparams.phy_chrom_width,h)!=h)
+           return true;
+   }
+   return false;
+}
+
+
+
+int Y4MPipeReader::PipeRead(uint8_t *buf, int len)
+{
+   int n, r;
+
+   r = 0;
+
+   while(r<len)
+   {
+      n = read(pipe_fd,buf+r,len-r);
+      if(n==0) return r;
+      r += n;
+   }
+   return r;
+}
+
 /**************************
  *
  * Derived class for options set from command line
@@ -270,7 +490,7 @@ void MPEG2EncCmdLineOptions::Usage()
 "    the --bitrate value sets an *upper-bound* video data-rate\n"
 "--output|-o pathname\n"
 "    Pathname of output file or fifo (REQUIRED!!!)\n"
-"--vcd-still-size|-T size\n"
+"--target-still-size|-T size\n"
 "    Size in KB of VCD stills\n"
 "--interlace-mode|-I num\n"
 "    Sets MPEG 2 motion estimation and encoding modes:\n"
@@ -298,6 +518,9 @@ void MPEG2EncCmdLineOptions::Usage()
 "--quantisation-reduction|-Q num\n"
 "    Max. quantisation reduction for highly active blocks\n"
 "    [0.0 .. 5] (default: 0.0)\n"
+"--quant-reduction-max-var|-X num\n"
+"    Luma variance below which quantisation boost (-Q) is used\n"
+"    [0.0 .. 2500.0](default: 0.0)\n"
 "--video-buffer|-V num\n"
 "    Target decoders video buffer size in KB (default 46)\n"
 "--video-norm|-n n|p|s\n"
@@ -340,7 +563,7 @@ void MPEG2EncCmdLineOptions::Usage()
 "--custom-quant-matrices|-K kvcd|tmpgenc|default|hi-res|file=inputfile|help\n"
 "    Request custom or userspecified (from a file) quantization matrices\n"
 "--unit-coeff-elim|-E num\n"
-"    Skip picture blocks satisfying which appear to carry little\n"
+"    Skip picture blocks which appear to carry little information\n"
 "    because they code to only unit coefficients. The number specifies\n"
 "    how aggresively this should be done. A negative value means DC\n"
 "    coefficients are included.  Reasonable values -40 to 40\n"
@@ -422,7 +645,7 @@ int MPEG2EncCmdLineOptions::SetFromCmdLine( int argc,	char *argv[] )
         { "video-buffer",      1, 0, 'V' },
         { "video-norm",        1, 0, 'n' },
         { "sequence-length",   1, 0, 'S' },
-        { "3-2-pulldown",      1, &vid32_pulldown, 1 },
+        { "3-2-pulldown",      0, &vid32_pulldown, 'p'},
         { "keep-hf",           0, 0, 'H' },
         { "reduce-hf",         1, 0, 'N' },
         { "sequence-header-every-gop", 0, &seq_hdr_every_gop, 1},
@@ -683,9 +906,9 @@ int MPEG2EncCmdLineOptions::SetFromCmdLine( int argc,	char *argv[] )
 			break;
 		case 'Q' :
 			act_boost = atof(optarg);
-			if( act_boost <-4.0 || act_boost > 10.0)
+			if( act_boost <-4.0 || act_boost > 5.0)
 			{
-				mjpeg_error( "-q option requires arg -4.0 .. 10.0");
+				mjpeg_error( "-q option requires arg -4.0 .. 5.0");
 				++nerr;
 			}
 			break;
